from typing import Any
from uuid import UUID

from niltype import Nil, Nilable
from th import PathHolder
from valera import ValidationResult, Validator
from valera.errors import ValueValidationError

from ._uuid_str_schema import UUIDStrSchema

__all__ = ("UUIDStrValidator",)


class UUIDStrValidator(Validator, extend=True):
    def visit_uuid_str(self, schema: UUIDStrSchema, *,
                       value: Any = Nil, path: Nilable[PathHolder] = Nil,
                       **kwargs: Any) -> ValidationResult:
        result = self._validation_result_factory()
        if path is Nil:
            path = self._path_holder_factory()

        if error := self._validate_type(path, value, str):
            return result.add_error(error)

        try:
            actual_value = UUID(value)
        except (TypeError, ValueError):
            error = ValueValidationError(path, value, schema.props.value)
            return result.add_error(error)

        if schema.props.value is not Nil:
            if actual_value != UUID(schema.props.value):
                error = ValueValidationError(path, value, schema.props.value)
                return result.add_error(error)

        return result



from typing import Any

from district42.representor import Representor
from niltype import Nil

from ._uuid_str_schema import UUIDStrSchema

__all__ = ("UUIDStrRepresentor",)


class UUIDStrRepresentor(Representor, extend=True):
    def visit_uuid_str(self, schema: UUIDStrSchema, *, indent: int = 0, **kwargs: Any) -> str:
        r = f"{self._name}.uuid_str"

        if schema.props.value is not Nil:
            r += f"({schema.props.value!r})"

        return r



from typing import Any, cast

from district42 import Props, SchemaVisitor
from district42 import SchemaVisitorReturnType as ReturnType
from district42.errors import (
    DeclarationError,
    make_already_declared_error,
    make_incorrect_max_error,
    make_incorrect_min_error,
    make_invalid_type_error,
)
from district42.types import Schema
from niltype import Nil, Nilable

__all__ = ("NumericSchema", "NumericProps",)


class NumericProps(Props):
    @property
    def value(self) -> Nilable[str]:
        return self.get("value")

    @property
    def min(self) -> Nilable[int]:
        return self.get("min")

    @property
    def max(self) -> Nilable[int]:
        return self.get("max")


class NumericSchema(Schema[NumericProps]):
    def __accept__(self, visitor: SchemaVisitor[ReturnType], **kwargs: Any) -> ReturnType:
        return cast(ReturnType, visitor.visit_numeric(self, **kwargs))

    def __call__(self, /, value: str) -> "NumericSchema":
        if not isinstance(value, str):
            raise make_invalid_type_error(self, value, (str,))

        if self.props.value is not Nil:
            raise make_already_declared_error(self)

        if (self.props.min is not Nil) or (self.props.max is not Nil):
            raise make_already_declared_error(self)

        try:
            int(value)
        except (TypeError, ValueError) as e:
            raise DeclarationError(str(e)) from None

        return self.__class__(self.props.update(value=value))

    def min(self, /, value: int) -> "NumericSchema":
        if not isinstance(value, int):
            raise make_invalid_type_error(self, value, (int,))

        if self.props.min is not Nil:
            raise make_already_declared_error(self)

        if (self.props.value is not Nil) and (value > int(self.props.value)):
            raise make_incorrect_min_error(self, self.props.value, value)

        return self.__class__(self.props.update(min=value))

    def max(self, /, value: int) -> "NumericSchema":
        if not isinstance(value, int):
            raise make_invalid_type_error(self, value, (int,))

        if self.props.max is not Nil:
            raise make_already_declared_error(self)

        if (self.props.value is not Nil) and (value < int(self.props.value)):
            raise make_incorrect_max_error(self, self.props.value, value)

        return self.__class__(self.props.update(max=value))



from uuid import uuid4

from baby_steps import given, then, when
from district42 import represent

from district42_exp_types.uuid_str import schema_uuid_str


def test_uuid_str_representation():
    with given:
        sch = schema_uuid_str

    with when:
        res = represent(sch)

    with then:
        assert res == "schema.uuid_str"


def test_uuid_str_value_representation():
    with given:
        value = str(uuid4())
        sch = schema_uuid_str(value)

    with when:
        res = represent(sch)

    with then:
        assert res == f"schema.uuid_str({value!r})"



from typing import Any

from niltype import Nil, Nilable
from th import PathHolder
from valera import ValidationResult, Validator
from valera.errors import MaxValueValidationError, MinValueValidationError

from ._numeric_schema import NumericSchema

__all__ = ("NumericValidator",)


class NumericValidator(Validator, extend=True):
    def visit_numeric(self, schema: NumericSchema, *,
                      value: Any = Nil, path: Nilable[PathHolder] = Nil,
                      **kwargs: Any) -> ValidationResult:
        result = self._validation_result_factory()
        if path is Nil:
            path = self._path_holder_factory()

        if error := self._validate_type(path, value, str):
            return result.add_error(error)

        if schema.props.value is not Nil:
            if error := self._validate_value(path, value, schema.props.value):
                return result.add_error(error)

        if schema.props.min is not Nil:
            if int(value) < int(schema.props.min):
                result.add_error(MinValueValidationError(path, value, schema.props.min))

        if schema.props.max is not Nil:
            if int(value) > int(schema.props.max):
                result.add_error(MaxValueValidationError(path, value, schema.props.max))

        return result



from typing import Any

from niltype import Nil
from revolt import Substitutor
from revolt.errors import make_substitution_error

from ._numeric_schema import NumericSchema

__all__ = ("NumericSubstitutor",)


class NumericSubstitutor(Substitutor, extend=True):
    def visit_numeric(self, schema: NumericSchema, *,
                      value: Any = Nil, **kwargs: Any) -> NumericSchema:
        result = schema.__accept__(self._validator, value=value)
        if result.has_errors():
            raise make_substitution_error(result)
        return schema.__class__(schema.props.update(value=value))



from typing import Any

from district42.representor import Representor
from niltype import Nil

from ._numeric_schema import NumericSchema

__all__ = ("NumericRepresentor",)


class NumericRepresentor(Representor, extend=True):
    def visit_numeric(self, schema: NumericSchema, *, indent: int = 0, **kwargs: Any) -> str:
        r = f"{self._name}.numeric"

        if schema.props.value is not Nil:
            r += f"({schema.props.value!r})"

        if schema.props.min is not Nil:
            r += f".min({schema.props.min!r})"

        if schema.props.max is not Nil:
            r += f".max({schema.props.max!r})"

        return r



from typing import Any
from uuid import UUID

from niltype import Nil, Nilable
from th import PathHolder
from valera import ValidationResult, Validator

from ._uuid_schema import UUIDSchema

__all__ = ("UUIDValidator",)


class UUIDValidator(Validator, extend=True):
    def visit_uuid(self, schema: UUIDSchema, *,
                   value: Any = Nil, path: Nilable[PathHolder] = Nil,
                   **kwargs: Any) -> ValidationResult:
        result = self._validation_result_factory()
        if path is Nil:
            path = self._path_holder_factory()

        if error := self._validate_type(path, value, UUID):
            return result.add_error(error)

        if schema.props.value is not Nil:
            if error := self._validate_value(path, value, schema.props.value):
                return result.add_error(error)

        return result



import pytest
from baby_steps import given, then, when
from th import PathHolder
from valera import validate
from valera.errors import (
    MaxValueValidationError,
    MinValueValidationError,
    TypeValidationError,
    ValueValidationError,
)

from district42_exp_types.numeric import schema_numeric


def test_numeric_type_validation():
    with when:
        result = validate(schema_numeric, "42")

    with then:
        assert result.get_errors() == []


def test_numeric_type_validation_error():
    with given:
        value = 42

    with when:
        result = validate(schema_numeric, value)

    with then:
        assert result.get_errors() == [TypeValidationError(PathHolder(), value, str)]


def test_numeric_value_validation():
    with given:
        value = "42"

    with when:
        result = validate(schema_numeric(value), value)

    with then:
        assert result.get_errors() == []


def test_numeric_value_validation_error():
    with given:
        expected_value = "42"
        actual_value = "43"

    with when:
        result = validate(schema_numeric(expected_value), actual_value)

    with then:
        assert result.get_errors() == [
            ValueValidationError(PathHolder(), actual_value, expected_value)
        ]


@pytest.mark.parametrize("value", ["1", "2"])
def test_numeric_min_value_validation(value: str):
    with when:
        result = validate(schema_numeric.min(1), value)

    with then:
        assert result.get_errors() == []


def test_numeric_min_value_validation_error():
    with given:
        min_value = 1
        actual_value = "0"

    with when:
        result = validate(schema_numeric.min(min_value), actual_value)

    with then:
        assert result.get_errors() == [
            MinValueValidationError(PathHolder(), actual_value, min_value)
        ]


@pytest.mark.parametrize("value", ["1", "2"])
def test_numeric_max_value_validation(value: str):
    with when:
        result = validate(schema_numeric.max(2), value)

    with then:
        assert result.get_errors() == []


def test_numeric_max_value_validation_error():
    with given:
        max_value = 1
        actual_value = "2"

    with when:
        result = validate(schema_numeric.max(max_value), actual_value)

    with then:
        assert result.get_errors() == [
            MaxValueValidationError(PathHolder(), actual_value, max_value)
        ]


@pytest.mark.parametrize("value", ["1", "2", "3"])
def test_numeric_min_max_value_validation(value: int):
    with when:
        result = validate(schema_numeric.min(1).max(3), value)

    with then:
        assert result.get_errors() == []


def test_numeric_min_max_greater_value_validation_error():
    with given:
        min_value = 1
        max_value = 3
        actual_value = "4"

    with when:
        result = validate(schema_numeric.min(min_value).max(max_value), actual_value)

    with then:
        assert result.get_errors() == [
            MaxValueValidationError(PathHolder(), actual_value, max_value)
        ]


def test_numeric_min_max_less_value_validation_error():
    with given:
        min_value = 1
        max_value = 3
        actual_value = "0"

    with when:
        result = validate(schema_numeric.min(min_value).max(max_value), actual_value)

    with then:
        assert result.get_errors() == [
            MinValueValidationError(PathHolder(), actual_value, min_value)
        ]



from district42 import register_type

from ._uuid_generator import UUIDGenerator
from ._uuid_representor import UUIDRepresentor
from ._uuid_schema import UUIDProps, UUIDSchema
from ._uuid_substitutor import UUIDSubstitutor
from ._uuid_validator import UUIDValidator

schema_uuid = register_type("uuid", UUIDSchema)

__all__ = ("UUIDSchema", "UUIDProps", "schema_uuid",
           "UUIDGenerator", "UUIDRepresentor", "UUIDSubstitutor", "UUIDValidator",)



from typing import Any

from blahblah import Generator
from niltype import Nil

from ._numeric_schema import NumericSchema

__all__ = ("NumericGenerator",)

INT_MIN = -(2 ** 63)
INT_MAX = 2 ** 63 - 1


class NumericGenerator(Generator, extend=True):
    def visit_numeric(self, schema: NumericSchema, **kwargs: Any) -> str:
        if schema.props.value is not Nil:
            return schema.props.value

        min_value = INT_MIN
        if schema.props.min is not Nil:
            min_value = int(schema.props.min)

        max_value = INT_MAX
        if schema.props.max is not Nil:
            max_value = int(schema.props.max)

        return str(self._random.random_int(min_value, max_value))



from typing import Callable
from uuid import uuid4

import pytest
from baby_steps import given, then, when
from th import PathHolder
from valera import validate
from valera.errors import TypeValidationError, ValueValidationError

from district42_exp_types.uuid_str import schema_uuid_str


@pytest.mark.parametrize("func", [str.lower, str.upper])
def test_uuid_str_type_validation(func: Callable[[str], str]):
    with given:
        value = func(str(uuid4()))

    with when:
        result = validate(schema_uuid_str, value)

    with then:
        assert result.get_errors() == []


def test_uuid_str_type_validation_error():
    with given:
        value = uuid4()

    with when:
        result = validate(schema_uuid_str, value)

    with then:
        assert result.get_errors() == [TypeValidationError(PathHolder(), value, str)]


@pytest.mark.parametrize(("actual_func", "expected_func"), [
    (str.lower, str.upper),
    (str.upper, str.lower),
])
def test_uuid_str_value_validation(actual_func: Callable[[str], str],
                                   expected_func: Callable[[str], str]):
    with given:
        value = str(uuid4())
        sch = schema_uuid_str(expected_func(value))

    with when:
        result = validate(sch, actual_func(value))

    with then:
        assert result.get_errors() == []


def test_uuid_str_value_validation_error():
    with given:
        expected_value = str(uuid4())
        actual_value = str(uuid4())

    with when:
        result = validate(schema_uuid_str(expected_value), actual_value)

    with then:
        assert result.get_errors() == [
            ValueValidationError(PathHolder(), actual_value, expected_value)
        ]


def test_uuid_str_invalid_value_validation_error():
    with given:
        expected_value = str(uuid4())
        actual_value = ""

    with when:
        result = validate(schema_uuid_str(expected_value), actual_value)

    with then:
        assert result.get_errors() == [
            ValueValidationError(PathHolder(), actual_value, expected_value)
        ]



from typing import Any

from niltype import Nil
from revolt import Substitutor
from revolt.errors import make_substitution_error

from ._uuid_schema import UUIDSchema

__all__ = ("UUIDSubstitutor",)


class UUIDSubstitutor(Substitutor, extend=True):
    def visit_uuid(self, schema: UUIDSchema, *, value: Any = Nil, **kwargs: Any) -> UUIDSchema:
        result = schema.__accept__(self._validator, value=value)
        if result.has_errors():
            raise make_substitution_error(result)
        return schema.__class__(schema.props.update(value=value))



from typing import Any, Callable
from unittest.mock import Mock, call

import pytest
from baby_steps import given, then, when
from blahblah import Generator, Random, RegexGenerator
from district42.types import Schema

from district42_exp_types.numeric import schema_numeric
from district42_exp_types.numeric._numeric_generator import INT_MAX, INT_MIN


@pytest.fixture()
def random_() -> Random:
    return Mock(Random, wraps=Random())


@pytest.fixture()
def regex_generator(random_: Random) -> RegexGenerator:
    return RegexGenerator(random_)


@pytest.fixture()
def generator(random_: Random, regex_generator: RegexGenerator) -> Generator:
    return Generator(random_, regex_generator)


@pytest.fixture()
def generate(generator: Generator) -> Callable[[Schema], Any]:
    def _generate(sch: Schema) -> Any:
        return sch.__accept__(generator)
    return _generate


def test_numeric_generation(*, generate, random_):
    with given:
        sch = schema_numeric

    with when:
        res = generate(sch)

    with then:
        assert isinstance(res, str)
        assert INT_MIN <= int(res) <= INT_MAX
        assert random_.mock_calls == [
            call.random_int(INT_MIN, INT_MAX)
        ]


def test_numeric_value_generation(*, generate, random_):
    with given:
        val = "42"
        sch = schema_numeric(val)

    with when:
        res = generate(sch)

    with then:
        assert res == val
        assert random_.mock_calls == []


def test_numeric_min_generation(*, generate, random_):
    with given:
        min_val = 1
        sch = schema_numeric.min(min_val)

    with when:
        res = generate(sch)

    with then:
        assert isinstance(res, str)
        assert int(res) >= min_val
        assert random_.mock_calls == [
            call.random_int(min_val, INT_MAX)
        ]


def test_numeric_max_generation(*, generate, random_):
    with given:
        max_val = 2
        sch = schema_numeric.max(max_val)

    with when:
        res = generate(sch)

    with then:
        assert isinstance(res, str)
        assert int(res) <= max_val
        assert random_.mock_calls == [
            call.random_int(INT_MIN, max_val)
        ]


def test_numeric_min_max_generation(*, generate, random_):
    with given:
        min_val, max_val = 1, 2
        sch = schema_numeric.min(min_val).max(max_val)

    with when:
        res = generate(sch)

    with then:
        assert isinstance(res, str)
        assert min_val <= int(res) <= max_val
        assert random_.mock_calls == [
            call.random_int(min_val, max_val)
        ]



from typing import Any
from uuid import uuid4

from blahblah import Generator
from niltype import Nil

from ._uuid_str_schema import UUIDStrSchema

__all__ = ("UUIDStrGenerator",)


class UUIDStrGenerator(Generator, extend=True):
    def visit_uuid_str(self, schema: UUIDStrSchema, **kwargs: Any) -> str:
        if schema.props.value is not Nil:
            return schema.props.value
        return str(uuid4())



from uuid import UUID, uuid4

from baby_steps import given, then, when
from blahblah import fake

from district42_exp_types.uuid_str import schema_uuid_str


def test_uuid_str_generation():
    with given:
        sch = schema_uuid_str

    with when:
        res = fake(sch)

    with then:
        assert isinstance(res, str)
        assert str(UUID(res)) == res


def test_uuid_str_value_generation():
    with given:
        val = str(uuid4())
        sch = schema_uuid_str(val)

    with when:
        res = fake(sch)

    with then:
        assert res == val



from uuid import uuid4

from baby_steps import given, then, when
from pytest import raises
from revolt import substitute
from revolt.errors import SubstitutionError

from district42_exp_types.uuid import schema_uuid


def test_uuid_substitution():
    with given:
        value = uuid4()
        sch = schema_uuid

    with when:
        res = substitute(sch, value)

    with then:
        assert res == schema_uuid(value)
        assert res != sch


def test_uuid_value_substitution():
    with given:
        value = uuid4()
        sch = schema_uuid(value)

    with when:
        res = substitute(sch, value)

    with then:
        assert res == schema_uuid(value)
        assert id(res) != id(sch)


def test_uuid_substitution_invalid_value_error():
    with given:
        sch = schema_uuid(uuid4())

    with when, raises(Exception) as exception:
        substitute(sch, str(uuid4()))

    with then:
        assert exception.type is SubstitutionError


def test_uuid_substitution_incorrect_value_error():
    with given:
        sch = schema_uuid(uuid4())

    with when, raises(Exception) as exception:
        substitute(sch, uuid4())

    with then:
        assert exception.type is SubstitutionError



from typing import Any

from niltype import Nil
from revolt import Substitutor
from revolt.errors import make_substitution_error

from ._uuid_str_schema import UUIDStrSchema

__all__ = ("UUIDStrSubstitutor",)


class UUIDStrSubstitutor(Substitutor, extend=True):
    def visit_uuid_str(self, schema: UUIDStrSchema, *,
                       value: Any = Nil, **kwargs: Any) -> UUIDStrSchema:
        result = schema.__accept__(self._validator, value=value)
        if result.has_errors():
            raise make_substitution_error(result)
        return schema.__class__(schema.props.update(value=value))



import pytest
from baby_steps import given, then, when
from district42 import represent

from district42_exp_types.numeric import schema_numeric


def test_numeric_representation():
    with given:
        sch = schema_numeric

    with when:
        res = represent(sch)

    with then:
        assert res == "schema.numeric"


@pytest.mark.parametrize(
    ("value", "expected_repr"),
    [
        ("42", "schema.numeric('42')"),
        ("0", "schema.numeric('0')"),
        ("-42", "schema.numeric('-42')"),
    ]
)
def test_numeric_value_representation(value: str, expected_repr: str):
    with given:
        sch = schema_numeric(value)

    with when:
        res = represent(sch)

    with then:
        assert res == expected_repr


def test_numeric_min_value_representation():
    with given:
        sch = schema_numeric.min(42)

    with when:
        res = represent(sch)

    with then:
        assert res == "schema.numeric.min(42)"


def test_numeric_max_value_representation():
    with given:
        sch = schema_numeric.max(42)

    with when:
        res = represent(sch)

    with then:
        assert res == "schema.numeric.max(42)"


def test_numeric_min_max_value_representation():
    with given:
        sch = schema_numeric.min(1).max(2)

    with when:
        res = represent(sch)

    with then:
        assert res == "schema.numeric.min(1).max(2)"


def test_numeric_min_max_with_value_representation():
    with given:
        sch = schema_numeric("2").min(1).max(3)

    with when:
        res = represent(sch)

    with then:
        assert res == "schema.numeric('2').min(1).max(3)"



from uuid import uuid4

from baby_steps import given, then, when
from district42.errors import DeclarationError
from pytest import raises

from district42_exp_types.uuid_str import UUIDStrSchema, schema_uuid_str


def test_uuid_str_declaration():
    with when:
        sch = schema_uuid_str

    with then:
        assert isinstance(sch, UUIDStrSchema)


def test_uuid_str_value_declaration():
    with given:
        value = str(uuid4())

    with when:
        sch = schema_uuid_str(value)

    with then:
        assert sch.props.value == value


def test_uuid_str_invalid_value_type_declaration_error():
    with given:
        value = uuid4()

    with when, raises(Exception) as exception:
        schema_uuid_str(value)

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == ("`schema.uuid_str` value must be an instance of 'str', "
                                        f"instance of 'UUID' {value!r} given")


def test_uuid_str_invalid_value_declaration_error():
    with when, raises(Exception) as exception:
        schema_uuid_str("")

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == "badly formed hexadecimal UUID string"


def test_uuid_str_already_declared_declaration_error():
    with given:
        value = str(uuid4())
        another_value = str(uuid4())

    with when, raises(Exception) as exception:
        schema_uuid_str(value)(another_value)

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == f"`schema.uuid_str({value!r})` is already declared"



from typing import Any
from uuid import UUID, uuid4

from blahblah import Generator
from niltype import Nil

from ._uuid_schema import UUIDSchema

__all__ = ("UUIDGenerator",)


class UUIDGenerator(Generator, extend=True):
    def visit_uuid(self, schema: UUIDSchema, **kwargs: Any) -> UUID:
        if schema.props.value is not Nil:
            return schema.props.value
        return uuid4()



from uuid import UUID, uuid4

from baby_steps import given, then, when
from th import PathHolder
from valera import validate
from valera.errors import TypeValidationError, ValueValidationError

from district42_exp_types.uuid import schema_uuid


def test_uuid_type_validation():
    with when:
        result = validate(schema_uuid, uuid4())

    with then:
        assert result.get_errors() == []


def test_uuid_type_validation_error():
    with given:
        value = str(uuid4())

    with when:
        result = validate(schema_uuid, value)

    with then:
        assert result.get_errors() == [TypeValidationError(PathHolder(), value, UUID)]


def test_uuid_value_validation():
    with given:
        value = uuid4()

    with when:
        result = validate(schema_uuid(value), value)

    with then:
        assert result.get_errors() == []


def test_uuid_value_validation_error():
    with given:
        expected_value = uuid4()
        actual_value = uuid4()

    with when:
        result = validate(schema_uuid(expected_value), actual_value)

    with then:
        assert result.get_errors() == [
            ValueValidationError(PathHolder(), actual_value, expected_value)
        ]



from uuid import UUID, uuid4

from baby_steps import given, then, when
from blahblah import fake

from district42_exp_types.uuid import schema_uuid


def test_uuid_generation():
    with given:
        sch = schema_uuid

    with when:
        res = fake(sch)

    with then:
        assert isinstance(res, UUID)


def test_uuid_value_generation():
    with given:
        val = uuid4()
        sch = schema_uuid(val)

    with when:
        res = fake(sch)

    with then:
        assert res == val



from district42 import register_type

from ._numeric_generator import NumericGenerator
from ._numeric_representor import NumericRepresentor
from ._numeric_schema import NumericProps, NumericSchema
from ._numeric_substitutor import NumericSubstitutor
from ._numeric_validator import NumericValidator

schema_numeric = register_type("numeric", NumericSchema)

__all__ = ("NumericSchema", "NumericProps", "schema_numeric",
           "NumericGenerator", "NumericRepresentor", "NumericSubstitutor", "NumericValidator",)



from district42 import register_type

from ._uuid_str_generator import UUIDStrGenerator
from ._uuid_str_representor import UUIDStrRepresentor
from ._uuid_str_schema import UUIDStrProps, UUIDStrSchema
from ._uuid_str_substitutor import UUIDStrSubstitutor
from ._uuid_str_validator import UUIDStrValidator

schema_uuid_str = register_type("uuid_str", UUIDStrSchema)

__all__ = ("UUIDStrSchema", "UUIDStrProps", "schema_uuid_str",
           "UUIDStrGenerator", "UUIDStrRepresentor", "UUIDStrSubstitutor", "UUIDStrValidator",)



from typing import Any, cast
from uuid import UUID

from district42 import Props, SchemaVisitor
from district42 import SchemaVisitorReturnType as ReturnType
from district42.errors import make_already_declared_error, make_invalid_type_error
from district42.types import Schema
from niltype import Nil, Nilable

__all__ = ("UUIDSchema", "UUIDProps",)


class UUIDProps(Props):
    @property
    def value(self) -> Nilable[UUID]:
        return self.get("value")


class UUIDSchema(Schema[UUIDProps]):
    def __accept__(self, visitor: SchemaVisitor[ReturnType], **kwargs: Any) -> ReturnType:
        return cast(ReturnType, visitor.visit_uuid(self, **kwargs))

    def __call__(self, /, value: UUID) -> "UUIDSchema":
        if not isinstance(value, UUID):
            raise make_invalid_type_error(self, value, (UUID,))

        if self.props.value is not Nil:
            raise make_already_declared_error(self)

        return self.__class__(self.props.update(value=value))



from baby_steps import given, then, when
from district42.errors import DeclarationError
from pytest import raises

from district42_exp_types.numeric import NumericSchema, schema_numeric


def test_numeric_declaration():
    with when:
        sch = schema_numeric

    with then:
        assert isinstance(sch, NumericSchema)


def test_numeric_value_declaration():
    with given:
        value = "42"

    with when:
        sch = schema_numeric(value)

    with then:
        assert sch.props.value == value


def test_numeric_invalid_value_type_declaration_error():
    with when, raises(Exception) as exception:
        schema_numeric(42)

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == ("`schema.numeric` value must be an instance of 'str', "
                                        "instance of 'int' 42 given")


def test_numeric_invalid_value_declaration_error():
    with when, raises(Exception) as exception:
        schema_numeric("a")

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == "invalid literal for int() with base 10: 'a'"


def test_numeric_already_declared_declaration_error():
    with when, raises(Exception) as exception:
        schema_numeric("42")("42")

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == "`schema.numeric('42')` is already declared"


def test_numeric_min_value_declaration():
    with given:
        min_value = 1

    with when:
        sch = schema_numeric.min(min_value)

    with then:
        assert sch.props.min == min_value


def test_numeric_invalid_min_value_type_declaration_error():
    with when, raises(Exception) as exception:
        schema_numeric.min("42")

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == ("`schema.numeric` value must be an instance of 'int', "
                                        "instance of 'str' '42' given")


def test_numeric_value_already_declared_min_declaration_error():
    with when, raises(Exception) as exception:
        schema_numeric.min(1)("42")

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == "`schema.numeric.min(1)` is already declared"


def test_numeric_min_value_already_declared_less_value_declaration_error():
    with given:
        sch = schema_numeric("42")

    with when, raises(Exception) as exception:
        sch.min(43)

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == (
            f"`{sch!r}` min value must be less than or equal to 42, 43 given"
        )


def test_numeric_min_value_already_declared_min_declaration_error():
    with when, raises(Exception) as exception:
        schema_numeric.min(1).min(2)

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == "`schema.numeric.min(1)` is already declared"


def test_numeric_max_value_declaration():
    with given:
        max_value = 2

    with when:
        sch = schema_numeric.max(max_value)

    with then:
        assert sch.props.max == max_value


def test_numeric_invalid_max_value_type_declaration_error():
    with when, raises(Exception) as exception:
        schema_numeric.max("42")

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == ("`schema.numeric` value must be an instance of 'int', "
                                        "instance of 'str' '42' given")


def test_numeric_value_already_declared_max_declaration_error():
    with when, raises(Exception) as exception:
        schema_numeric.max(100)("42")

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == "`schema.numeric.max(100)` is already declared"


def test_numeric_max_value_already_declared_greater_value_declaration_error():
    with given:
        sch = schema_numeric("42")

    with when, raises(Exception) as exception:
        sch.max(41)

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == (
            f"`{sch!r}` max value must be greater than or equal to 42, 41 given"
        )


def test_numeric_max_value_already_declared_max_declaration_error():
    with when, raises(Exception) as exception:
        schema_numeric.max(2).max(1)

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == "`schema.numeric.max(2)` is already declared"


def test_numeric_min_max_value_declaration():
    with given:
        min_value, max_value = 1, 2

    with when:
        sch = schema_numeric.min(min_value).max(max_value)

    with then:
        assert sch.props.min == min_value
        assert sch.props.max == max_value


def test_numeric_min_max_with_value_declaration():
    with given:
        value = "2"
        min_value, max_value = 1, 3

    with when:
        sch = schema_numeric(value).min(min_value).max(max_value)

    with then:
        assert sch.props.value == value
        assert sch.props.min == min_value
        assert sch.props.max == max_value



import pytest
from baby_steps import given, then, when
from pytest import raises
from revolt import substitute
from revolt.errors import SubstitutionError

from district42_exp_types.numeric import schema_numeric


def test_numeric_substitution():
    with given:
        sch = schema_numeric

    with when:
        res = substitute(sch, "42")

    with then:
        assert res == schema_numeric("42")
        assert res != sch


def test_numeric_value_substitution():
    with given:
        value = "42"
        sch = schema_numeric(value)

    with when:
        res = substitute(sch, value)

    with then:
        assert res == schema_numeric(value)
        assert id(res) != id(sch)


def test_numeric_substitution_invalid_value_error():
    with given:
        sch = schema_numeric("42")

    with when, raises(Exception) as exception:
        substitute(sch, 42)

    with then:
        assert exception.type is SubstitutionError


def test_numeric_substitution_incorrect_value_error():
    with given:
        sch = schema_numeric("42")

    with when, raises(Exception) as exception:
        substitute(sch, "50")

    with then:
        assert exception.type is SubstitutionError


@pytest.mark.parametrize("value", ["2", "1"])
def test_numeric_substitution_min(value: str):
    with given:
        sch = schema_numeric.min(1)

    with when:
        res = substitute(sch, value)

    with then:
        assert res == schema_numeric(value).min(1)
        assert res != sch


def test_numeric_substitution_min_value_error():
    with given:
        sch = schema_numeric.min(1)

    with when, raises(Exception) as exception:
        substitute(sch, "0")

    with then:
        assert exception.type is SubstitutionError


@pytest.mark.parametrize("value", ["0", "1"])
def test_numeric_substitution_max(value: str):
    with given:
        sch = schema_numeric.max(1)

    with when:
        res = substitute(sch, value)

    with then:
        assert res == schema_numeric(value).max(1)
        assert res != sch


def test_numeric_substitution_max_value_error():
    with given:
        sch = schema_numeric.max(1)

    with when, raises(Exception) as exception:
        substitute(sch, "2")

    with then:
        assert exception.type is SubstitutionError


@pytest.mark.parametrize("value", ["1", "2", "3"])
def test_numeric_substitution_min_max(value: str):
    with given:
        sch = schema_numeric.min(1).max(3)

    with when:
        res = substitute(sch, value)

    with then:
        assert res == schema_numeric(value).min(1).max(3)
        assert res != sch


@pytest.mark.parametrize("value", ["0", "4"])
def test_numeric_substitution_min_max_value_error(value: str):
    with given:
        sch = schema_numeric.min(1).max(3)

    with when, raises(Exception) as exception:
        substitute(sch, value)

    with then:
        assert exception.type is SubstitutionError



from uuid import uuid4

from baby_steps import given, then, when
from district42 import represent

from district42_exp_types.uuid import schema_uuid


def test_uuid_representation():
    with given:
        sch = schema_uuid

    with when:
        res = represent(sch)

    with then:
        assert res == "schema.uuid"


def test_uuid_value_representation():
    with given:
        value = uuid4()
        sch = schema_uuid(value)

    with when:
        res = represent(sch)

    with then:
        assert res == f"schema.uuid({value!r})"



from uuid import uuid4

from baby_steps import given, then, when
from district42.errors import DeclarationError
from pytest import raises

from district42_exp_types.uuid import UUIDSchema, schema_uuid


def test_uuid_declaration():
    with when:
        sch = schema_uuid

    with then:
        assert isinstance(sch, UUIDSchema)


def test_uuid_value_declaration():
    with given:
        value = uuid4()

    with when:
        sch = schema_uuid(value)

    with then:
        assert sch.props.value == value


def test_uuid_invalid_value_type_declaration_error():
    with given:
        value = str(uuid4())

    with when, raises(Exception) as exception:
        schema_uuid(value)

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == ("`schema.uuid` value must be an instance of 'UUID', "
                                        f"instance of 'str' {value!r} given")


def test_uuid_already_declared_declaration_error():
    with given:
        value = uuid4()
        another_value = uuid4()

    with when, raises(Exception) as exception:
        schema_uuid(value)(another_value)

    with then:
        assert exception.type is DeclarationError
        assert str(exception.value) == f"`schema.uuid({value!r})` is already declared"



from typing import Any

from district42.representor import Representor
from niltype import Nil

from ._uuid_schema import UUIDSchema

__all__ = ("UUIDRepresentor",)


class UUIDRepresentor(Representor, extend=True):
    def visit_uuid(self, schema: UUIDSchema, *, indent: int = 0, **kwargs: Any) -> str:
        r = f"{self._name}.uuid"

        if schema.props.value is not Nil:
            r += f"({schema.props.value!r})"

        return r



from typing import Any, cast
from uuid import UUID

from district42 import Props, SchemaVisitor
from district42 import SchemaVisitorReturnType as ReturnType
from district42.errors import (
    DeclarationError,
    make_already_declared_error,
    make_invalid_type_error,
)
from district42.types import Schema
from niltype import Nil, Nilable

__all__ = ("UUIDStrSchema", "UUIDStrProps",)


class UUIDStrProps(Props):
    @property
    def value(self) -> Nilable[str]:
        return self.get("value")


class UUIDStrSchema(Schema[UUIDStrProps]):
    def __accept__(self, visitor: SchemaVisitor[ReturnType], **kwargs: Any) -> ReturnType:
        return cast(ReturnType, visitor.visit_uuid_str(self, **kwargs))

    def __call__(self, /, value: str) -> "UUIDStrSchema":
        if not isinstance(value, str):
            raise make_invalid_type_error(self, value, (str,))

        try:
            UUID(value)
        except ValueError as e:
            raise DeclarationError(str(e)) from None

        if self.props.value is not Nil:
            raise make_already_declared_error(self)

        return self.__class__(self.props.update(value=value))
